module Miniform
  class Field
    def initialize(builder, name, options, &block)
      @builder = builder
      @name = name
      @options = options
      @block = block
    end

    def render
      if should_omit_label?
        @block.call + help_block
      else
        @builder.form_group(@name) do
          if vertical?
            label + @block.call + help_block
          else
            label + input_wrapper { @block.call + help_block  }
          end
        end
      end
    end

    private

      def vertical?
        @builder.is_a?(VerticalFormBuilder)
      end

      def should_omit_label?
        @options.key?(:label) && !@options[:label]
      end

      def label
        if vertical?
          @builder.label(@name, @options[:label])
        else
          @builder.label(@name, @options[:label], class: "col-md-3")
        end
      end

      def help_block
        unless @options[:error] == false
          @builder.help_block(@name)
        end
      end

      def input_wrapper(&block)
        @builder.content_tag(:div, class: "col-md-9", &block)
      end
  end
end
