module Miniform
  class FormBuilder < ActionView::Helpers::FormBuilder
    def form_group(*names, &block)
      classes = %w[ form-group ]
      classes << "has-error" if names.any? { |n| errors(n) }
      content_tag :div, class: classes, &block
    end

    def label(name, *args)
      options = options_with_class(args.extract_options!, "control-label")
      super(name, *args, options)
    end

    %w[ text_field phone_field text_area email_field password_field ].each do |method|
      define_method method do |name, *args|
        input_options = options_with_class(args.extract_options!, "form-control")
        field_options = extract_field_options!(input_options)

        Field.new(self, name, field_options) do
          super(name, *args, input_options)
        end.render
      end
    end

    def select(name, choices, options = {}, html_options = {})
      field_options = extract_field_options!(options)
      html_options = options_with_class(html_options, "form-control")

      Field.new(self, name, field_options) do
        super(name, choices, options, html_options)
      end.render
    end

    def collection_select(method, collection, value_method, text_method, options = {}, html_options = {})
      field_options = extract_field_options!(options)
      html_options = options_with_class(html_options, "form-control")

      Field.new(self, method, field_options) do
        super(method, collection, value_method, text_method, options, html_options)
      end.render
    end

    def datetime_select(method, options = {}, html_options = {})
      field_options = extract_field_options!(options)
      html_options = options_with_class(html_options, "form-control")

      Field.new(self, method, field_options) do
        super(method, options, html_options)
      end.render
    end

    def help_block(name)
      if errors(name)
        content_tag(:div, errors(name), class: "help-block")
      else
        nil
      end
    end

    def errors(name)
      object.errors[name].any? && object.errors[name].join(", ")
    end

    def content_tag(*args, &block)
      @template.content_tag(*args, &block)
    end

    def fields_for(*args, &block)
      options = args.extract_options!
      options[:builder] ||= Miniform::FormBuilder
      super(*args, options, &block)
    end

    private

      def extract_field_options!(options)
        options.extract!(:label, :error)
      end

      def options_with_class(options, css_class)
        options.dup.tap do |o|
          o[:class] = Array(o[:class]) << css_class
        end
      end
  end
end
